/**
 * @file  AE3Channel.h
 * @brief Schnittstelle fuer die Klasse CAE3Channel.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_)
#define AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "AE3Buffer.h"
#include "AE3Modbus.h"

// Datensatzbefehle und Antworten

// Datensatzanfrage
#define MB_CMD_READ_RECORD              0x40  /* Datensatz senden */

#define MB_ANSW_NO_RECORD               0xC0  /* kein Datensatz vorhanden */
#define MB_ANSW_OFFLINE_RECORD          0xC1  /* Offline Datensatz */
#define MB_ANSW_OLD_OFFLINE_RECORD      0xC3  /* wiederholt gesendeter Offline Datensatz */
#define MB_ANSW_ONLINE_RECORD           0xE1  /* Online Datensatz */
#define MB_ANSW_OLD_ONLINE_RECORD       0xE3  /* wiederholt gesendeter Online Datensatz */

// Datensatzquittierung
#define MB_CMD_ACK_RECORD               0x48  /* Quittung fuer Datensatz */

#define MB_ANSW_ACK_OFF_RECORD	        0xC8  /* Quittung erhalten (fuer Offline-Datensatz) */
#define MB_ANSW_OLD_ACK_OFF_RECORD      0xCA  /* wiederholt gesendete Quittung */
#define MB_ANSW_ACK_ON_RECORD           0xE8  /* Quittung erhalten (fuer Online-Datensatz) */
#define MB_ANSW_OLD_ACK_ON_RECORD       0xEA  /* wiederholt gesendete Quittung */
#define MB_ANSW_TIMEOUT_ACK_RECORD      0xE9  /* Quittung kam nach Timeout (fuer Online-Datensatz) */
#define MB_ANSW_OLD_TIMEOUT_ACK_RECORD  0xEB  /* wiederholte Quittung nach Timeout */

// Datensatzloeschbefehl
#define MB_CMD_DEL						0x49  /* Loescht alle Datensaetze. */

// alte Befehle des AEIII+
#define CMD_READ		0x31
#define CMD_READ_AGAIN	0x32
#define CMD_DEL			0x33

// alte Antworten des AEIII+
#define ANSW_DAT_NOANSW 0x31	// Keine Antwort vom PC erwartet
#define ANSW_DAT_ANSW	0x71	// Antwort vom PC erwartet
#define ANSW_NODAT		0xB1	// Keine Datem im Geraet


#include "AE3ErrorDef.h"

//		Adresse des Pointers der im EEPROM liegt und aud das Flash zeigt
//#define P_DAT_STRUCT2	0x0000	/* Datensatzbeschreibung */
//#define P_KONFIG		0x0004	/* Konfiguration */
//#define P_START_RING	0x0008	/* Startadresse Ringspeicher */
//#define P_END_RING		0x000C	/* Endadresse Ringspeicher */
//#define P_READ_RING		0x0010	/* Readpointer Ringsp. */
//#define P_WRITE_RING	0x0014	/* Writepointer Ringspeicher */
//#define P_WRITE_R_1		0x0018	/* Writepointer R. letzte Schreibperation (bezogen auf Datensatz) */
//#define P_WRITE_R_2		0x001C	/* Writepointer R. vor letzte Schreibperation (bezogen auf Datensatz) */
//#define P_WRITE_R_3		0x0020	/* Writepointer R. vor vor letzte Schreibperation (bezogen auf Datensatz) */
#define P_LIST_STRUCT	0x0024	/* Startadresse Listenbeschreibung */
//#define P_LIST_0		0x0028	/* Adresse der Daten der Liste Nr. 0 im Flash */
								/* Die weiteren Adressen sind nachfolgend gespeichert */
								/* z.B. Adresse von Liste Nr. 4  P_LIST_0+4*sizeof(ULONG) */
/* #define P_LIST_19     0x0074*/  /* Pointer auf letzte Liste */
#define P_DAT_STRUCT	0x0080	/* Kopie vonP_DAT_STRUCT Fehler von ATMega zu umgehen */

// Datafox
//#define RAM_DF_END			0x77f00 // letzte Speicherzelle im Datafox RAM bei AE III mit 512kB
//#define MEMD_START			0x8000		// Startadd. Daten im RAM bei AE III mit 512kByte
#define MB_CONTROL_STOP		0x48000000
#define MB_RECORD_AVAILABLE 0x48000001

#define DISPLAY_MEMORY		0x49000000
#define INFOTEXT_MEMORY		0x4B000000

#define MB_SET_TIME			0x43000000	// Offset fuer Uhr schreiben
#define MB_GET_VERSION		0x42000000	// Offset fuer Version lesen
#define MB_GET_TIME			0x44000000	// Offset fuer Uhr lesen

#define MB_START_EEP		0x40000000	// Startadresse EEP-ROM
//#define EEP_SIZE_SU			0x200		/* 512Byte fuer Setup-Daten */
#define MB_SERIEN_MEMORY	0x4A000800

#define MB_START_CONFIG		0x4A000000	// Startadresse AE-Konfiguration
#define MB_START_IOMODULE	0x4C000000  // Startadresse IOModule - Struktur
#define MB_DIGITAL_IO		0x4D000000  // Startadresse DigitalMDE - Struktur fuer IOModul
#define MB_DIGITAL_MDE		0x4E000000  // Startadresse DigitalMDE - Struktur fuer Interne Digitaleingaenge
#define MB_MDE_EK			(FLASH_KONFIG_START + FLASH_SIZE_SU) // Startadresse Eingabeketten - Strukturen fuer MDE

// Tabellen fuer Zutrittskontrolle
#define MB_ENTRANCE_TABLE_0 0x4F000000
#define MB_ENTRANCE_TABLE_1 0x4F100000
#define MB_ENTRANCE_TABLE_2 0x4F200000
#define MB_ENTRANCE_TABLE_3 0x4F300000

// Eingabeketten fuer Zutrittskontrolle
#define MB_ENTRANCE_EK      (FLASH_KONFIG_START + FLASH_SIZE_SU + EXTENDED_SIZE_SU) // Startadresse Eingabeketten - Strukturen fuer Zutrittskontrolle

// Datenaufbau: 
// 1tes Byte:
// bit(7) gesetzt ist setzen. nicht gesetzt ist lesen
// bit(6) .. bit(0) ist Nummer.
// 2tes Byte: frei bei Relais und Laenge fuer GlobVars
// 3tes und 4tes Byte:
// Dauer fuer Relais, bei GlobVar muss nicht vorhanden sein.
#define MB_RELAY            0x50000000  // Startadresse Relais
#define MB_GLOBAL_VAR       0x51000000  // Startadresse Globale Variable

// 32 Byte Zeichenabfrage fuer Firmwareversion
#define MB_VERSION          0x5D000000 
// Starten der Datenuebertragung und des Bootloaderresets
#define MB_BOOTLOAD         0x5E000000 
#define MB_RESTART          0x5C000000 // Neustart PZE Master IV

// lesen/schreiben der Speicherbereiche
#define MB_ACCESS_SRAM		0xF0000000
#define MB_ACCESS_EEPROM 	0xF1000000
#define MB_ACCESS_FLASH		0xF2000000
#define MB_ACCESS_PROGRAM	0xF3000000

class CAE3Channel : 
	public CAE3Buffer, 
	public CAE3Modbus  
{
public:
	// Konstruktor fuer Kanalinitialisierung
	CAE3Channel(unsigned char ucNum);
	virtual ~CAE3Channel();

	// Wechsel der Busnummer bedingt verschiedene einstellungen
	virtual int ChangeBusNum(unsigned char ucNewBusNum);

	int CheckDevice(unsigned char ucBusNum, int *piError, int iPollRetry);
	int Restart(unsigned char ucBusNum, int *piError);
	int CheckCmd(unsigned char ucBusNum, int *piError);

	// Senden einer Bestaetigungsmeldung an das AEIII+
	int SendMessage(unsigned char ucBusNum, unsigned char ucDelayTime,
		unsigned char ucAction,	unsigned char ucAudio, char *szMessage, 
		int iLength, int *piError);

	// Senden eines Infotextes fuer Geraetestatus.
	int SendInfotext(unsigned char ucBusNum, char *szMessage, int iLength, int *piError);

	// Abgleich von Datum und Uhrzeit auf Betriebsystem
	int SetDateTime(unsigned char ucBusNum, unsigned char *pucDateTime, int *piError);
	// Ermitteln von Datum und Uhrzeit 
	int GetDateTime(unsigned char ucBusNum, unsigned char *pucDateTime, int *piError);

	int SetGlobVar(unsigned char ucBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int *piError);
	int GetGlobVar(unsigned char ucBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int iValueLength, int *piError);

	int GetTableDescription(unsigned char ucBusNum,	unsigned char ucList, int *piError);
	int GetListName(unsigned char ucBusNum, unsigned char ucList, char *name, int *piError);

	// Datensatzfunktionen, Variable und Puffer fuer letzten gelesenen.
	int m_iRecordLength, m_iRecordError;
	unsigned char m_pucRecordBuf[MAX_MB_PAKET];
	unsigned char m_pucTmpRecordBuf[MAX_MB_PAKET];
	// Pruefen ob min. ein Datensatz im AEIII+ vorhanden.
	int CheckData(unsigned char ucBusNum, int *piError);
	// Lesen eines einzelnen Datensatzes. Online zuerst.
	int ReadRecord(unsigned char ucBusNum, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	// Quittiert den zuletzt gelesenen Datensatz um naechsten anzufordern.
	int QuitRecord(unsigned char ucBusNum, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	// Loescht alle Datensaetzte im AEIII+
	int ClearData(unsigned char ucBusNum, int *piError);

	void ResetWithError(int err);

	int DFReadRecord(int deviceID, char *buf, int *length, int *error);
	int DFQuitRecord(int deviceID, int *error);

	// Speicher initialisieren
	void ClearMem();

#ifdef _WIN32
	// Alte Datensatzfunktionen
	int ReadDatas(unsigned char ucBusNum, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	int ReReadDatas(unsigned char ucBusNum, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	int OldCollectData(unsigned char ucBusNum, int *piError);
	int GetDatensatz(unsigned char *pucBuffer, int *piLength, int *piError);
#endif

	// Laden der Importierten Listendaten
	int LoadListData(unsigned char ucBusNum, int *piError);
	int LoadNewListData(unsigned char ucBusNum, int *piError);
	// Laden der Importierten Listendaten fuer Zutrittskontrolle
	int LoadEntranceListData(unsigned char ucBusNum, int iListNum, int *piError);

	int UpLoadSetupData(unsigned char ucBusNum, int *piError);
	int DownLoadSetupData(unsigned char ucBusNum, int *piError);
	int CheckComVersion(unsigned char ucBusNum, int *piError);
	int GetSerialNum(unsigned char ucBusNum, int *piSerialNum, int *piError);

	int GetVersionFirmware(unsigned char ucBusNum, char *pVersion, int *piError);
	int GetVersionFirmwareFromFile(char *pFileName, char *pVersion, int *piError);

	int CheckFileDisplay(CDFFileDisplay *pFileDisplay, int *piError);
	int UploadFileDevice(unsigned char ucBusNum, int *piError);
	
	// Relais setzten/ruecksetzen und zustand abfragen.
	int CloseRelay(unsigned char ucBusNum, int iNum, int iTime, int *piError);
	int GetRelayState(unsigned char ucBusNum, int iNum, int *piState, int *piTime, int *piError);
	int OpenRelay(unsigned char ucBusNum, int iNum, int *piError);

	// Kommunikationsversion des letzten Geraets aus dem daten per ComCollectData
	// ausgelesen wurden.
	unsigned short m_usLastCollectVersion; 
	// Kommunikationsversion des letzten Geraets fuer Setup lesen oder schreiben usw...
	unsigned short m_usLastDeviceVersion;
	// Anzahl der letzten Ermittlungsversuche halten.
	int m_iDevicePollRetry;
};

#endif // !defined(AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_)

