/**
 * @file  AE3Modbus.h
 * @brief Schnittstelle fuer die Klasse CAE3Modbus.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_)
#define AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Channel.h"
#include "AE3ErrorDef.h"

#define MAX_MB_PAKET			0xFF //0x5E0	// max. MOD-Bus Datenpaket
#define MAX_MB_PAKET_HEADER		13
#define MAX_MB_PAKET_DATA		(MAX_MB_PAKET - MAX_MB_PAKET_HEADER)

#define MAX_WBUF	MAX_MB_PAKET
#define MAX_RBUF	MAX_MB_PAKET

#define MB_READ					0x21	// bis zu 242 Byte Daten lesen
#define MB_WRITE				0x22	// bis zu 242 Byte Daten schreiben

#define MB_ADD_OFFLINE			0xFE	

extern const int g_iErrorMap[];

// Modbusprotokoll fuer AEIII+ verwalten.
class CAE3Modbus : public CChannel  
{
public:
	CAE3Modbus(unsigned char ucChannelNum);
	virtual ~CAE3Modbus();

public:
	int ExeCallBack();
	void SetCBFunction(int (*pFunction)());
	unsigned char GetBusNum();
	virtual int ChangeBusNum(unsigned char ucNewBusNum);
	
	void ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val);
	void UIntToBinMSBFirst(unsigned char *Buf,unsigned int Val);
	int BinToUIntMSBFirst(unsigned char *Buf);

protected:
	unsigned char m_pucRxBuf[MAX_MB_PAKET];		// MOD-Bus Read-Buffer
	unsigned char m_pucTxBuf[MAX_MB_PAKET];		// MOD-Bus Write-Buffer
	unsigned char m_ucBusNum;		// aktuelle Busnummer	

	// Sende und Empfang von Modbus - Kommando
	int Command(unsigned char *pucTxBuf, int iTxCount, 
				unsigned char *pucRxBuf, int *piRxCount, 
				int *piError);

	// Schreiben an Geraetespeicheradresse
	int WriteMem(unsigned long Add, int Cnt, unsigned char *Buf, int *piError, int MaxB = MAX_MB_PAKET_DATA);
	int WriteXByte (unsigned long Add, int Cnt, unsigned char *Buf, int *piError);
	// Lesen von Geraetespeicheradresse
	int ReadMem(unsigned long Add, int Cnt, unsigned char *Buf, int *piError);
	int ReadXByte (unsigned long Add, int Cnt, unsigned char *Buf, int *piError);

	// CRC16 - Funktionen
	void MkMBPaket(unsigned char *buf, int cnt);
	int CheckMBPaket(unsigned char *buf, int cnt);

	int (*CBFunction)();
};

#endif // !defined(AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_)

