/** 
 * @file Channel.h
 * @brief Schnittstelle fuer die Klasse CChannel.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CHANNEL_H__1A0DF5B2_A206_44D8_9359_8B91D1E4EAD2__INCLUDED_)
#define AFX_CHANNEL_H__1A0DF5B2_A206_44D8_9359_8B91D1E4EAD2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Klasse fuer Multimaster.
#include "Com.h"

#define COM_LOG_FILENAME "DfCom_%i.log"

/// Basisklasse fuer pure Kommunikationsaufgaben.
class CChannel  
{
public:
	/// Konstruktor fuer Initialisierung der Kanalklasse.
	CChannel(unsigned char ucNum);
	/// Destruktor
	virtual ~CChannel();

	/// Ermitteln der zugewiesenen Kanalnummer.
	UCHAR GetChannelNum();

	/// Bereinigen der Schnittstelle.
	void ClearCom();

	/// Typ des Kommunikationskanals
	enum enumChannelType { Undef, Serial, Socket, Class };
	// Typ des Kanals ermitteln
	enumChannelType GetChannelType() { return m_ChannelType; }

	/// Ermitteln des Schnittstellenhandle.
	HANDLE GetComPort();

	/// Setzen der Schnittstellenparameter.
	int SetComPort(HANDLE hCom, int iType, int iComValue, int iTimeOut);

	// Alte Funktionen
	int ComUseHandle(HANDLE hnd);
	int ComReleaseHandle();

	/// Oeffnen der Schnittstelle fuer RS232 / RS485 Verbindung.
	int  OpenSerial(char *ComPort, int iBaudRate, int iTimeOut);
	/// Oeffnen der Schnittstelle fuer TCP/IP.
	int  OpenSocket(char *RemoteIP, int iPortNum, int iTimeOut);
	/// Oeffnen der Class - Schnittstelle fuer RS232 / RS485 Verbindung.
	int  OpenClass(int iAddress, int iType, char *szComString, int iComValue, int iTimeOut);

	/// Ermitteln ob Schnittstelle geoeffnet.
	int IsOpen();

	/// Schliessen der Schnittstelle.
	void Close();

	/// Setzten des Logdateinamen.
	void SetLogFileName(const char *szLogFileName);
	
	/// Log anschalten.
	inline void SetLogOn()  
	{ m_bLogIsOn = true; m_Com.SetLogOn(); }
	
	/// Log ausschalten.
	inline void SetLogOff() 
	{ m_bLogIsOn = false; m_Com.SetLogOff(); }

protected:
	// Eigenen Handle nicht mit ReleaseHandle freigeben.
	bool m_bOwnHandle;

	int  m_iComPortTimeOut;
	enumChannelType m_ChannelType;  // Typ der Schnittstelle
	HANDLE m_hCom;                  // Handle serielle   Schnittstelle
	SOCKET m_hSocket;               // Handle TCP_Socket Schnittstelle

	// Klasse fuer Erweitertes - Modbus - Protokoll, Multimaster
public:	
	CCom m_Com;

protected:
	int  SetSerialComState(HANDLE hComPort, int iBaudRate, int iTimeOut);
	int  SerialOpen(char *szComPort, int iBaudRate, int iTimeOut);
	void SerialClose(void);
	int  SocketOpen(char *szRemoteIP, int iPortNo, int iTimeOut);
	void SocketClose(void);

protected:
	// Lesen
	int  SocketRead(char *pcInputBuf, int iInputLen, unsigned long *pdwReadLen);
	int  SerialRead(char *pcInputBuf, int iInputLen, unsigned long *pdwReadLen);
public:
	int  Read(void *pBuf, int iLength, unsigned long *pdwReadLen);
	
protected:
	// Schreiben
	int  SocketWrite(const char *pcOutputBuf, int iOutputLen, unsigned long *pdwWriteLen);
	int  SerialWrite(const char *pcOutputBuf, int iOutputLen, unsigned long *pdwWriteLen);
public:
	int  Write(void *pBuf, int iLength, unsigned long *pdwWriteLen);

protected:
	bool m_bLogIsOn;
	char m_szLogFileName[MAX_PATH];

#ifndef TRACE_DEBUG
	void ComLog(const char *szFormat, ...);
#else
	#define ComLog TRACE
#endif

private:
	// zugewiesene Kanalnummer
	unsigned char m_ucNum;
#ifndef _WIN32
	struct termio termio_s;
	struct flock  flock_s;
#endif
};

#endif // !defined(AFX_CHANNEL_H__1A0DF5B2_A206_44D8_9359_8B91D1E4EAD2__INCLUDED_)

