#ifndef SETUP_H
#define SETUP_H

#pragma pack(1)

// ByteOrder
#define BO_MOT	   1	// MSB zuerst
#define BO_INT	   0	// LSB zuerst

#define EEP_SIZE			0x1000   // 4kByte EEPROM 
#define EEP_SIZE_SU			0x0200   // 512 Byte fuer Setupdaten 
#define EEP_SIZE_PT			0x2C     // Bereich fuer Pointer 
#define FLASH_BAS			0x0FC000 // Startadresse im Flash fuer Setup-Daten 
#define FLASH_SIZE_SU		0x4000   // 16kByte fuer Setup-Daten 
#define CONFIG_SIZE_SU		0x64     // 100Byte fuer Konfigurationsdaten
#define EXTENDED_SIZE_SU	0x4000   // 16kByte erweiterter Speicher fuer IOModul usw.. 
#define FLASH_KONFIG_START	0x0FC000 // Startadresse fuer Konfig-Daten im Flash 
#define FLASH_LIST_START	0x0E0000 // Abhaengig von Setup-Var. "Speicher"  Speicher = 0 
#define FLASH_LIST_START_NEW 0x180000 // Ab 04.00.23 (ComVersion 5.2) release (244kByte)
#define FLASH_SIZE_LIST		0x018000 // Speicher = 0 
#define FLASH_SIZE_LIST_NEW 0x03D000 // Ab 04.00.23 (ComVersion 5.2) release (244kByte)
#define DATEN_SIZE			0x0E0000 // max. Speicher fuer Daten 
#define MAX_FLASH_MEM		0x100000 // Buffer fuer Flash-Daten AE

// Pointer im EEPROM 
// ------------------
// Adresse des Pointers der im EEPROM liegt und auf das Flash zeigt
#define P_VERSION		0x0000	// Versionsnummer des Setup.
#define P_KONFIG		0x0004	// Konfiguration 

/*
// Pointer fuer Datenspeicherung 
#define P_START_RING	0x0008	// Startadresse Ringspeicher 
#define P_END_RING		0x000C	// Endadresse Ringspeicher 
#define P_READ_RING		0x0010	// Readpointer Ringsp. 
#define P_WRITE_RING	0x0014	// Writepointer Ringspeicher 
#define P_WRITE_R_1		0x0018	// Writepointer R. letzte Schreibperation (bezogen auf Datensatz) 
#define P_WRITE_R_2		0x001C	// Writepointer R. vor letzte Schreibperation (bezogen auf Datensatz) 
#define P_WRITE_R_3		0x0020	// Writepointer R. vor vor letzte Schreibperation (bezogen auf Datensatz) 
*/
#define P_HEADLINE      0x0008 // Kopfzeile im Hauptmenue (17 Zeichen + Nullterminie)
//FREE          0x001A - 0x0023

#define P_LIST_STRUCT	0x0024	// Startadresse Listenbeschreibung 
#define P_LIST_0		0x0028	// Adresse der Daten der Liste Nr. 0 im Flash 
								// Die weiteren Adressen sind nachfolgend gespeichert 
								// z.B. Adresse von Liste Nr. 4  P_LIST_0+4 * sizeof(ULONG) 
// #define P_LIST_19     0x0074    // Pointer auf letzte Liste 
#define P_DAT_STRUCT	0x0080	// Kopie vonP_DAT_STRUCT Fehler von ATmega zu umgehen 

#define P_FLASH_SIZE_SU    0x0084  // Groesse der Flashdaten
#define P_MDE_EK_SIZE      0x0088  // Groesse der MDE - Eingabeketten
#define P_ENTRANCE_EK_SIZE 0x008C  // Groesse der Zutrittskontroll - Eingabeketten
#define P_LIST_KEY_FIELD   0x0090  // Fuer alle 20 Moeglichen LSBs je ein Byte fuer ein Schluesselfeld reservieren.
#define P_AUTO_KEY         0x00A5  // Tastenumschaltung
#define P_SETUP_VERSION    0x00E5  // Ab: 03.01.30.00; Es werden in vier Byte die Versionsnummer
                                   //     des Setup mit Buildnumber abgelegt
#define P_DEVICE_TYPE      0x00E9  // Ab: 03.01.31.00; Es wird in einem Byte der Geraetetyp
                                   //     abgespeichert, fuer den dieses Setup gespeichert wird.
                                   //     DT_AEIII, DT_AEIV
#define P_FREE             0x00EA

//  Daten im EEPROM
// Bereich im EEPROM der nicht vom Setup-Programm ueberschrieben wird 
#define LCD_KONTRAST	0x0200	// Kontrastwert LCD 
#define SERIEN_NUMMER	0x0202	// 2 Byte 
#define UHR_KORREKTUR	0x0204	// Korrekturwert fuer RTC um Ganggenauigkeit zu optimieren 
#define ALPHA_TIMER_VAL	0x0206	// Wartezeit fuer Eingabe von Buchstaben ueber 10ner-Tastatur 


// Daten fuer AE-Konfiguration in ConfigMem
#define BUS_NUM					0x00
#define LCD_LIGHT				0x02
#define ONLINE_WAIT				0x04
#define SERVER_ONLINE			0x06
#define SYSTEMMENU_PASSWORD		0x08
#define MDE_DIVIDER				0x0A
#define MDE_TIMEOUT				0x0C
#define MDE_ZYKLUS				0x0E
#define TERMINAL_MODE			0x10
#define TRANSPONDER_READER		0x12
#define ACTION_E1_UP			0x14
#define ACTION_E1_DOWN			0x16
#define ACTION_E2_UP			0x18
#define ACTION_E2_DOWN			0x1A
#define ONOFF_DEACTIVATE		0x1C
#define INFO_TABLEAU 			0x1E
#define UNIQUE5B_Flags			0x20
#define UNIQUE5B				0x21

// Frueherer MDE - Bereich fuer Transponder verwenden; SW03.01.32
#define TP_GLOBAL               0x22
#define TP_ACCESS_CONTROL       0x28
#define HITAG_AUTOINC_FLAGS     0x2E // Flags fuer Autoinkrement bei Karten schreiben
#define HITAG_AUTOINC_VALUE     0x2F // 16BitWert um den Inkrementiert wird
#define TRANSPONDER_CUT_FLAGS   0x31 // Flags fuer Zuschneidefunktion der Legic/Mifare Leser
#define TRANSPONDER_CUT_FROM    0x32 // Start fuer Ausschneiden von Transponderwert bei Legic & Mifare
#define TRANSPONDER_CUT_LENGTH  0x33 // Laenge des auszuschneidenen Transponderwerts 
#define IOMODULE_ACTIVE			0x34 // 1 ist aktiv, 0 ist deaktiviert. 
#define TRANSPONDER		        0x36 // HI: SegmentNum, LO: 0:Unique, 1:Hightech, 2:Legic (Highbit gibt Segmentverwendung an, naechstes bit fuer -nur Seriennummer lesen-)
#define KEY_SOUND               0x38 // Tastenton
// 0x3A // Display - Kontrast
// 0x3C // ? frei
#define LED_PERMANENT			0x3E // Letzten FTaste durch blinken abzeigen, wie AE3
// 0x40 // TimeOut fuer MB
// 0x42 .. 0x45 // IP - Address
#define HITAG_INITIAL_VALUE_3   0x42 // Startwert fuer Kartensegment 3 beschreiben
#define ENABLE_ENTRANCE         0x46 // Zutrittskontolle aktivieren
#define EXT_READER              0x47 // externer Leser verwenden
#define DOOR_SIGNAL_TIME        0x48 // Dauer fuer Tueroeffner
#define DOOR_MONITORING         0x49 // Tuerueberwachung aktivieren
#define DOOR_ALARM_TIME         0x4A // Alarmzeit fuer geoeffnete Tuer
#define ENTRANCE_MAIN_MENU      0x4B // Ueberpruefen von Zutritt im Hauptmenue.
#define ENABLE_CUT              0x4C
#define CUT_AT                  0x4D
#define CUT_TO                  0x4E
//#define HITAG_PREFIX            0x4F // Kartenpraefix fuer Hitag
//#define HITAG_PREFIX_SEGMENT    0x53 // Segment in/aus welches dieses geschrieben/gelesen wird
#define HITAG_INITIAL_VALUE_1   0x54 // Startwert fuer Kartensegment 1 beschreiben
//#define HITAG_PREFIX_TYPE		0x58 // Typ fuer Wert des Kartenpraefix
#define EVER_BIG_WATCH			0x59 // Grosze Uhr immer anzeigen
#define TIMEOUT_PZE				0x5A // Timeout nach Eingaben zur Pruefung der Tastenumschalttabelle
#define MDE_LAMP_FLAGS			0x5B // Flags fuer die (rote) Lampensteuerung der MDE
#define HITAG_FIX_LENGTH        0x5C // Feste Stellenanzahl fuer Kartenwert; 0 => aus; wird links mit 0en aufgefuellt
#define HITAG_PASSWORD          0x5D // Passwort fuer Kartenprogrammiermenue; 0 => aus; 16bit
#define HITAG_INITIAL_VALUE_2   0x5F // Startwert fuer Kartensegment 2 beschreiben
// 1Byte frei

// ALT
// Datensatz im Flash incl. Verzeigerung und CRC 
//	Aufbau:	
//		Byte 0:		Laenge des Datensatzes einschlieslich CRC (x + 11) 
//		Byte 1..4:	Pointer auf naechsten Datensatz bzw. freien Speicher 
//		Byte 5..8:	Pointer auf vorhergehenden Datensatz (NULL -> kein Vorgaenger) 
//		Byte 9..9 + (x - 1): x-Byte Daten 
//		Byte 9 + x, 10 + x : CRC nach Modbus-Norm 

// Offset
#define DS_LEN		0	// Laenge Datensatz 
#define DS_NP		1	// naechster Datensatz 
#define DS_PP		5	// vorhergehender Datensatz 
#define DS_DAT		9
#define DS_CRC_X	10	// Offset CRC - x 
#define DS_KOPF		9
#define DS_KOPF_CRC	11	// Anzahl Byte fuer Datensatz ohne Nutzdaten 

// max. Laenge Datensatz ohne Verwaltungsdaten 
#define MAX_DATENSATZ	241 /*(255 - DS_KOPF_CRC)*/
#define MAX_FELD		25	// max. Felder in einem Datensatz 
#define MAX_AN_FELD		21	// 20 Zeichen + 0 

#define MAX_UNTERPUNKTE 100	    // max. Unterpunkte im Baum (Konfig, Datensatzb. und Listenb.) 

#define MAX_DATENSAETZE	70577	// max. Anzahl Datensaetze im Timeboy 

// Datensatzbeschreibung (nur Nutzdaten ohne Verwaltungsinformationen) 
//	P_DAT_STRUCT im EEPROM zeigt auf den Begin der Datensatzbeschreibung im Flash 
//  Ab dem Begin ist ein Array von max. "MAX_DATENSATZ_BESCHREIBUNG" Zeigern fuer jede Datensatzbeschreibung 
//	Die Pos. des Zeigers (ab 0) ist die Nummer der Datensatzbeschr. 
//	Die Zeiger zeigen auf je eine Struktur "TDatensatzBeschreibung" 
#define MAX_DATENSATZ_BESCHREIBUNG 20 // auch max. Listen 
#define MAX_DS_NAME		           17 // max. Zeichen fuer Datensatz/Feld-Name incl. abschliesende Null 

typedef struct StructDatensatzBeschreibung {
	UCHAR Felder;	              // Anzahl Felder im Datensatz 
	UCHAR Name[MAX_DS_NAME];      // Name der Datensatzbeschreibung 
	ULONG P;	                  // Pointer auf Feldbeschreibungen (Typ "TDatensatzFeld") 
} TDatensatzBeschreibung;

typedef struct StructDatensatzFeld {
	UCHAR Typ;		              // Aufzaehlung Typ des Feldes 
	UCHAR Cnt;		              // Laenge des Feldes inc. anbschlieende '\0' bei Strings 
	UCHAR Name[MAX_DS_NAME];	  // Name des Feldes 
} TDatensatzFeld;

//	Feldtypen
#define TF_ULONG	1	// 4 Byte 
#define TF_DT		2	// Date und Time 7 Byte -> 4 Byte Date und 3 Byte Time 
#define TF_NUM		3	// wie Typ TF_A_NUM, es sind jedoch nur Ziffern erlaubt 
#define TF_A_NUM	4	// Alpha-Numerisch ASCII Laengenangabe notwendig 
#define TF_DATE		5	// 4 Byte Y100,Y,M,D 
#define TF_TIME		6	// 3 Byte H,Mi,S 
//  Laenge der Feldtypen 
#define TF_ULONG_LEN	4	// 4 Byte 
#define TF_DATE_LEN		4	// 4 Byte Y100,Y,M,D 
#define TF_TIME_LEN		3	// 3 Byte H,Mi,S 
#define TF_DT_LEN		7	// Date und Time 7 Byte -> 4 Byte Date und 3 Byte Time 

// Allgemeine Def. fuer Konfiguration 
#define MAX_MENUE_TXT	17	// Laenge Texte im Menue incl. NULL 
#define MAX_S_FELD_TXT  17	// Laenge fuer Text S-Felder incl. NULL 
#define MAX_DATEN_LEN	21	// max. Laenge fuer ein Datensatzfeld 

// Konfiguration 
//	Bediensteuerung 
//	Die Bedienung wird durch die Konfig-Struktur gesteuert. 
//	Die Konfig-S. ist durch Pointer strukturiert. 
//  Die Bedienung arbeitet diese Srtructur ab, wie ein Interpreter. 
//	"P_KONFIG" zeigt auf den ersten Eintrag der Verketteten Strukturen.(wird meist nach "On" ausgefuehrt) 

// Typen der Bedienung 
#define TB_MENUE		1
#define TB_DATENSATZ	2
#define TB_SYSTEM		3 // Sytemmenue 
#define TB_KONFIG_ROOT	4
#define TB_LISTE		5 // Liste anzeigen (nicht speichern) 
#define TB_CLR_LAST		6 // Letzten Datensatz loeschen 

// Arten der Menueauswahl 
#define TBMA_NORM		0x01	// Auswahl ueber Pfeil- und F-Tasten 
#define TBMA_BARCODE	0x02	// Auswahl ueber Barcode EK1->Eintrag 1; EK2->Eintrag 2 ... 

// Einlesemetoden fuer Datensaetze 
#define TBDA_NORM	0		// Standard 
#define TBDA_LIST	1		// Schon eingegebene Daten werde in Listenform angezeigt 

// Einlesemethoden fuer Felder 
#define TBFE_NORM				1
#define TBFE_GET_TIME			2  // aktuelle Datum Uhrzeit aus AEIII+ 
#define TBFE_LIST_AUSWAHL		3  // aus Liste auswaehlen 
#define TBFE_KONSTANTE			4  // Konstante in den Datensatz schreiben 
#define TBFE_BESTAETIGUNG		5  // Bestaetigung abfragen (fuellt kein Datenfeld) 
#define TBFE_GLOB_VAR			6  // aus Glog.-Var. kopieren 
#define TBFE_SERIENNUMMER		7  // Seriennummer des AEIII+ uebernehmen 
#define TBFE_ANALOG_WERT		8  // Analogen Messwert in Feld uebernehmen. 
#define TBFE_DIGITAL_WERT		9  // Digitalen Messwert in Feld uebernehmen. 
#define TBFE_ZAEHLER_WERT		10 // Zaehlerstand in Feld uebernehmen.
#define TBFE_ENTRANCE_PNR		11 // Eingelesene Personalnummer von AEIII+ uebernehmen.
#define TBFE_ENTRANCE_STATUS	12 // Status von Zutritt uebernehmen. 
#define TBFE_FUNC				13 // Funktionswert uebernehmen
#define TBFE_SPS_WERT			14 // Wert von SPS uebernehmen
#define TBFE_GPRS_ALIVE_COUNTER	15 // Aufgelaufene Alivedatensaetze zaehlen
#define TBFE_FIRMWARE_VERSION   16 // Firmware-Version uebernhemen
#define TBFE_ONLINE_STATUS      17 // online/offline als -Server Online- Zustand uebernehmen

#define TBFE_DEBUG				255 // Einlesemethode fuer interne Zwecke


// Flags fuer TP TBTransponder.tbModeX
#define BDTP_ACTIVE                 0x80
//  lesen / schreiben
#define BDTP_WRITE                  0x40

#define BDTP_DEC                    0x00
#define BDTP_HEX                    0x10
#define BDTP_ASCII                  0x20
// Untere 4bit sind Anzahl feste stellen

// FeldEinlesenFlag
#define FEF_APPEND_TO_FIELD     0x01
#define FEF_APPEND_TO_GV        0x02


// Quellen fuer Eingabe von Daten (Maske in Var. Flag) 
#define NFF_TASTATUR	0x80
#define NFF_BARCODE		0x40
#define NFF_TRANSPONDER 0x20
#define NFF_NO_RET		0x01	// kein CR bei Barcodeeingabe notwendig 
#define NFF_ANZEIGEN	0x0E	// 3 Bit 8 Werte fuer Anzeigedauer wenn kein Return notwendig 

// Verhalten am Ende einer Eingabekette oder bei ESC 
#define EV_MENUE        0  // zurueck ins Menue 
#define EV_OFFSET_ESC   1  // Offset fuer Feldnummer bei Eingabekettenfeld 
#define EV_OFFSET_CR    2  // Offset fuer Feldnummer bei Eingabekette 
#define EV_NEXT_SET   255  // naechste Eingabekette ausfuehren. 

// Ein Varhalten 
#define EVERH_AKTUELL		0 // Nach Ein auf der aktuellen Pos vor Aus bleiben 
#define EVERH_MENUE			1 // Nach Einschalten zurueck in das Menue
							// >1 -> Eintrag aus Hauptmenue starten 

// Flags Datensatz Einlesen 
#define BDEF_W_UEBER			0x01 // Bei Schleifen bereits eingegebene Werte als 
									 // Vorgabe fuer Eingabe uebernehmen 
#define BDEF_NICHT_SPEICHERN	0x02 // Datensatz nicht speichern (nur Globale Variable) 
#define BDEF_ZUSATZ_RETURN		0x04 // Abfrage "Speichern ?" am Ende der Eingabekette 
#define BDEF_RUECKSPRUNG_MENUE	0x08 // Nach Einlesen eines Datensatzes Ruecksprungmenue anzeigen 
#define BDEF_ENTRANCE_ACCESS    0x10 // Zutrittskontrolle vor Abarbeitung der Eingabekette durchfuehren.
#define BDEF_RELAIS             0x20 // Relais fuer bestimmte Zeit schalten.
#define BDEF_RECORD_DOUBLE      0x40 // Eingabekette vor evntl. Eingabe von Werten ausfuehren, danach "Normale" Abarbeitung. 

// Flags fuer Menueeintrag
#define BDME_KLEMMEN_MDE        0x01  // MDE: Taste geklemmt! 
#define BDME_ANMELDEN_MDE       0x02  // MDE: Anmelde-Datensatz, der letzte 'normale' wird danach wiederholt! 

// Flags Barcode fuer TBExListAuswahl.SelBarcode 
#define BDLB_SEL_BARCODE		0x01	// Selection in Liste mit Barcode freigeben 
#define BDLB_BESTAETIGEN		0x02	// Ausgewaehlten Eintrag bestaetigen mit Ret 
#define BDLB_AUSBLENDEN			0x04    // Automatisches Ausblenden der Liste nach n Sekunden. 
// von Bit(3) .. Bit(7)                 // Zeit bis Liste auszublenden ist. 
// Flags Barcode fuer TBExListAuswahl.Flags 
#define BDLB_SEL_TRANSPONDER    0x01	// Selection in Liste mit Transponder freigeben 
#define BDLB_SEL_TASTATUR       0x02	// Selection in Liste mit Tastatur freigeben 

//#ifdef AE3
//typedef struct StructBedien {
//	volatile UCHAR Typ;// Typ der Bedienung Siehe TB_..... 
//	volatile UCHAR d1,d2;	// Reserve fuer Erweiterung 
//	volatile ULONG P;		// Pointer auf Struktur welche durch "Typ" ausgewaehlt wird 
//} TBedien;
//#else
volatile typedef struct StructBedien {
	volatile UCHAR	Typ;	// Typ der Bedienung Siehe TB_..... 
	volatile UCHAR  d1,d2;	// Reserve fuer Erweiterung 
	volatile ULONG	P;		// Pointer auf Struktur welche durch "Typ" ausgewaehlt wird 
} TBedien;
//#endif

typedef struct StructListAnz {
	UCHAR	ListNum;	// Nummer der Liste die angezeigt wird 
	UCHAR	Name[MAX_MENUE_TXT];
	UCHAR   Dummy[8];	// Reserve
} TBListAnz;

typedef struct StructMenue {
	UCHAR	Flags;	    // Flags wie Zutrittskontrollabfrage aktiviert usw..
	UCHAR	EintragCnt;	// Anzahl Eintraege im Menue 
	UCHAR	Name1[MAX_MENUE_TXT];		// Ueberschrift1 des Menues 
	UCHAR	Name2[MAX_MENUE_TXT];		// Ueberschrift2 des Menues 
	UCHAR   Dummy[8];	// Reserve
	ULONG	P;			// Pointer auf Liste der Eintraege 
} TBMenue;

typedef struct StructKonfigRoot {

	unsigned short	AutoAus;// Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfogt 
							// 0 -> kein Automatische abschaltung 
	UCHAR	EintragCnt;	// Anzahl Eintraege in KonfigRoot 
	UCHAR	EinVerhalten;	// Verhalten nach Einschalten 
	UCHAR	Dummy[8];	// Reserve 
	ULONG	P;			// Pointer auf Pointerarry mit "EintragCnt" Eintraegen 
} TBKonfigRoot;

typedef struct StructMenueEintrag {
	UCHAR	Name[MAX_MENUE_TXT];	// Menue Text 
	UCHAR	EintragCnt;	// Anzahl Eintraege in KonfigRoot 
	UCHAR	PassL;		// Passwort Low-Byte 
	UCHAR	PassH;		// Passwort High-Byte  0 -> kein Passwort 
	UCHAR	Flags;		// Parameter fuer MDE 
	UCHAR	Dummy[5];	// Reserve 
	ULONG	P;			// Pointer auf Aktion nach Auswahl des Menuepunktes 
} TBMenueEintrag;

typedef struct StructDatensatzEinlesen {
	UCHAR	DsNum;		// Nummer der Datensatzbeschreibung (ab 0) 
	UCHAR	Felder;		// Anzahl Felder (mu mit Datensatzbeschreibung uebereinstimmen) 
	UCHAR	CrV;		// Verhalten bei Enter-Taste 
	UCHAR	Name[MAX_MENUE_TXT];		// Ueberschrift 
	UCHAR	Typ;		// Bedien-Typ des Einlesens 
	UCHAR	Flags;		// Diverse Flags Definiert als BDEF_.... 
	UCHAR	K1Txt;		// Text in Feld K2 Bit 0...3 Nummer-Globale-Variable 
						//                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. 
	UCHAR	EscV;		// Verhalten bei ESC-Taste
	UCHAR	dummy;      // war mal Flags fuer Master - Slave - Felder.
	UCHAR   ucRelaisNum;// Nummer des anzusprechenden Relais.
	unsigned short usRelaisTime; // Zeit fuer die das ausgewaehlte Relais schliesst.
	UCHAR	Dummy;	    // Reserve 
	ULONG	P;			// Pointer auf Liste der Strukturen fuer Einlesen der Felder 
						// (Anzahl und Reihenfolge mu mit Datensatzbeschreibung uebereinstimmen) 
} TBDatensatzEinlesen;

// 51 Byte
typedef struct StructExBestaetigung {
	UCHAR	Text1[MAX_MENUE_TXT]; // Hinweistext 
	UCHAR	Text2[MAX_MENUE_TXT]; // Hinweistext 
	UCHAR	Text3[MAX_MENUE_TXT]; // Hinweistext
} TBExBestaetigung;

#define FECT_NONE            0x00
#define FECT_FORMAT_STRING   0x01
#define FECT_FORMAT_GV       0x02
// 21 Byte
typedef struct StructCompare {
	UCHAR type;                        // Typ fuer die Vergleichsart; Formatstring || GV
	UCHAR formatGV;                    // Wenn type GV: Bit 0..3: Arraynummer der GV, anderen Bits fuer Comboboxindex
	UCHAR formatString[MAX_MENUE_TXT]; // Wenn type Formatstring: Der Formatstring.
	UCHAR negate;                      // Verzweigen wenn Vergleich NICHT zutraf.
	UCHAR jumpDestination;             // Sprungziel, als Feldindex (0..n), von Eingabekette.
} TBCompare;

typedef struct StructTransponder {

	UCHAR tpSegment1;             // Segment 1 fuer Tag lesen / schreiben
	UCHAR tpMode1;                // Modus 1 fuer Tag lesen / schreiben 
	UCHAR tpSegment2;             // Segment 2 fuer Tag lesen / schreiben
	UCHAR tpMode2;                // Modus 2 fuer Tag lesen / schreiben 
	UCHAR tpSegment3;             // Segment 3 fuer Tag lesen / schreiben
	UCHAR tpMode3;                // Modus 3 fuer Tag lesen / schreiben 

} TBTransponder;

// 30 Byte
typedef struct StructExNumFeld {
	UCHAR	Flags;	// Flags z.B. Einlesen ueber Barcode erlaubt 
	TBTransponder tag;
	UCHAR temp[6];
	UCHAR	KonstAscii[MAX_MENUE_TXT];
	TBCompare compare;
} TBExNumFeld;

// 15 Byte
typedef struct StructExListAuswahl {
	UCHAR	Sel;	     // Flag Listenauswahl mit Selection 
	UCHAR	SelDFeld;	 // Feld im Datensatz fuer Selection (nur wenn "Sel" = TRUE) 
	UCHAR	SelLFeld;	 // Feld in der Liste nach dem selectiet wird (nur wenn "Sel" = TRUE) 
	UCHAR	List;	     // Nummer der Liste 
	UCHAR	AuswahlFeld; // Feld in der Liste nach dem der Benutzer auswaehlt 
	UCHAR	DatFeld;	 // Feld in der Liste das im Datensatz gespeichert wird 
	UCHAR	SelBarcode;
	UCHAR   Flags;        // Eingabequellen: Transponder, Tastatur
	UCHAR   NoValueCheck; // Wert nicht in Liste
	UCHAR   tempa[5];     // Tempfeld um hinter TBExNumFeld zu kommen.
	UCHAR   MapFKey;      // Die ersten zehn Listeneintraege auf Funktionstasten legen.
	UCHAR	SelDFeld2;    // wie SelDFeld
	UCHAR	SelLFeld2;    // wie SelLFeld
	UCHAR	SelNoEntry;   // Verhalten wenn kein Eintrag vorhanden.
	UCHAR   usedFontSize; // Zu verwendente Schriftgroesze bei Listenanzeige
	UCHAR	temp[11];
	TBCompare compare;
} TBExListAuswahl;

typedef struct StructFeldEinlesen {
	UCHAR	Typ;		// Typ der Einlesemetode "TBFE..." 
	UCHAR	DFNum;		// Nummer des Datensatzfeldes (ab 0) 
	UCHAR	EscV;		// Verhalten bei ESC-Taste 
	UCHAR	Name[MAX_MENUE_TXT]; // Ueberschrift 
	UCHAR	K2Txt;		// Text in Feld K2 Bit 0..3 Nummer-Globale-Variable 
						//                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. 
	UCHAR	GlobVar;	// Feld-Wert in Globale Var. kopieren (Aufbau wie K2Txt) 
	UCHAR	flags;	    // FEF_... 
	UCHAR	BarcodeVon;	// Bereich aus Barcode ausschneiden 1 -> erstes Zeichen 0 -> deaktiviert 
	UCHAR	BarcodeBis;	
	UCHAR	InputTypCnt; // Bei Methode Normal Feldlaenge und Typ angeben.
	UCHAR	MessVar;
	UCHAR	DigitalIO;
	union Ex {
		TBExBestaetigung	Bestaetigung;
		TBExNumFeld			NumFeld;
		TBExListAuswahl		ListAuswahl;
	} Ex;
} TBFeldEinlesen;

typedef struct StructSetupAllgemein {
	UCHAR	SFeld[8][MAX_S_FELD_TXT];
	ULONG	P;			// Pointer auf Root StructBedien 
} TBAllgemein;

// Include wird von DLL und Setup gemeinsam verwendet.
#include "ExtendedFlash.h"

#pragma pack()

#endif
