/**
 * @file  ATmega128.cpp
 * @brief Implementierung der Klasse CATmega128.
 */
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "ATmega128.h"

#ifndef _WIN32
#include <string.h>
#endif

CATmega128::CATmega128()
{
	Reset();
}

CATmega128::~CATmega128()
{
}

void CATmega128::Reset()
{
	// Speicher initialisieren
	memset(m_eeprom, 0xFF, sizeof(m_eeprom));
	memset(m_program, 0xFF, sizeof(m_program));
	memset(m_eepromUse, 0, sizeof(m_eepromUse));
	memset(m_programUse, 0, sizeof(m_programUse));
}

void CATmega128::SetEepromUsebit(UINT32 address)
{
	m_eepromUse[address / 32] |= (1 << (address % 32));
}

void CATmega128::SetProgramUsebit(UINT32 address)
{
	m_programUse[address / 32] |= (1 << (address % 32));
}

UINT8 CATmega128::GetEepromUsebit(UINT32 address)
{
	return (m_eepromUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

UINT8 CATmega128::GetProgramUsebit(UINT32 address)
{
	return (m_programUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

UINT32 CATmega128::GetProgramFirstAddress()
{
	UINT32 address, firstAddress = 0;
	for (address=0; address < ATMEGA128_SIZE_APPLICATION; address++) {

		if (GetProgramUsebit(address)) {

			firstAddress = address;
			break;
		}
	}

	return firstAddress;
}

UINT32 CATmega128::GetProgramLastAddress()
{
	UINT32 address, lastAddress = 0;
	for (address=0; address < ATMEGA128_SIZE_APPLICATION; address++) {

		if (GetProgramUsebit(address)) {

			lastAddress = address;
		}
	}

	return lastAddress;
}

UINT32 CATmega128::GetEepromFirstAddress()
{
	UINT32 address, firstAddress = 0;
	for (address=0; address < ATMEGA128_SIZE_EEPROM; address++) {

		if (GetEepromUsebit(address)) {

			firstAddress = address;
			break;
		}
	}

	return firstAddress;
}

UINT32 CATmega128::GetEepromLastAddress()
{
	UINT32 address, lastAddress = 0;
	for (address=0; address < ATMEGA128_SIZE_EEPROM; address++) {

		if (GetEepromUsebit(address)) {

			lastAddress = address;
		}
	}

	return lastAddress;
}

