#ifndef _DATAFLASH_H_
#define _DATAFLASH_H_

#include "typedef.h"

// 2MByte / 16megabit Speicher
//#define DATAFLASH_SIZE		0x200000
#define DATAFLASH_SIZE		0x10000
 
class CDataFlash 
{
public:
	CDataFlash();
	virtual ~CDataFlash();

	// Ruecksetzen
	void Reset();

	// EEPROM
	inline UINT8 *GetMemDirect(UINT32 address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE) 
		{ ASSERT(0); return 0; }
		return &m_mem[address]; 
	}
	inline UINT8 GetMem(UINT32 address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE) 
		{ ASSERT(0); return 0; }
		return m_mem[address]; 
	}
	inline UINT32 GetMemRange(UINT32 address, UINT8 *value, UINT32 size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_SIZE)
		{ ASSERT(0); return 0; }
		for (UINT32 i=address; i < (address + size); i++) {
		
			value[i - address] = m_mem[i];
		}
		return (i - address);
	}
	inline void SetMem(UINT32 address, UINT8 value) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE)
		{ ASSERT(0); return; }
		m_mem[address] = value; 
		SetMemUsebit(address + m_baseAddress);
	}
	inline void SetMemRange(UINT32 address, UINT8 *value, UINT32 size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_SIZE)
		{ ASSERT(0); return; }
		for (UINT32 i=address; i < (address + size); i++) {
		
			m_mem[i] = value[i - address];
			SetMemUsebit(i + m_baseAddress);
		}
	}

	// Helper
	void SetBaseAddress(UINT32 base) { m_baseAddress = base; };
	UINT32 GetBaseAddress() { return m_baseAddress; };
	void SetMemUsebit(UINT32 address);
	UINT8 GetMemUsebit(UINT32 address);
	UINT32 GetMemFirstAddress();
	UINT32 GetMemLastAddress();

protected:
	UINT32 m_baseAddress;
	UINT8 m_mem[DATAFLASH_SIZE];
	UINT32 m_memUse[DATAFLASH_SIZE / 32];
};

#endif
