/**
 * @file  IntelHex.h
 * @brief Schnittstelle fuer die Klasse CIntelHex.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _INTELHEX_H_
#define _INTELHEX_H_

// Datentypen
#include "typedef.h"

// Grunddaten und Grundoffsets
#define INTELHEX_DATA_COUNT 20  // Anzahl Datenbyte pro Datensatz
#define INTELHEX_START_SIGN ':' // Startzeichen eines Datensatzes
#define INTELHEX_START    0  // start of record
#define INTELHEX_COUNT    1  // byte count
#define INTELHEX_ADDRESS  3  // load address
#define INTELHEX_TYPE     7  // record type
#define INTELHEX_DATA     9  // record data
#define INTELHEX_CHECKSUM 7  // record checksum

// Standard - Datensatztypen
#define INTELHEX_TYPE_DAT 0x00 // Data record
#define INTELHEX_TYPE_EOF 0x01 // End-of-file record
#define INTELHEX_TYPE_ESA 0x02 // Extended Segment Address Record
#define INTELHEX_TYPE_SSA 0x03 // Start Segment Address Record
#define INTELHEX_TYPE_ELA 0x04 // Extended Linear Address Record
#define INTELHEX_TYPE_SLA 0x05 // Start Linear Address Record

// Datei der Hardwaredaten
#define INTELHEX_TYPE_HARDWARE_HEADER   0x10
#define INTELHEX_TYPE_HARDWARE_EEPROM   0x11
#define INTELHEX_TYPE_HARDWARE_FLASH    0x12

// Datei der Anzeigedaten
#define INTELHEX_TYPE_DISPLAY_HEADER  0x20
#define INTELHEX_TYPE_DISPLAY_FONT    0x21
#define INTELHEX_TYPE_DISPLAY_GRAFIC  0x22

// Datei der Geraetedaten
#define INTELHEX_TYPE_DEVICE_HEADER   0x30
#define INTELHEX_TYPE_DEVICE_EEPROM   0x31
#define INTELHEX_TYPE_DEVICE_FLASH    0x32
#define INTELHEX_TYPE_DEVICE_CRYPT    0x33

// Datei der EVA - Daten
#define INTELHEX_TYPE_DOCUMENT_HEADER    0x40
#define INTELHEX_TYPE_DOCUMENT_EEPROM    0x41
#define INTELHEX_TYPE_DOCUMENT_FLASH     0x42
#define INTELHEX_TYPE_DOCUMENT_XFLASH    0x43
#define INTELHEX_TYPE_DOCUMENT_CONFIG    0x44
#define INTELHEX_TYPE_DOCUMENT_PATH      0x45
#define INTELHEX_TYPE_DOCUMENT_PATH2     0x46


class CIntelHex
{
public:
	CIntelHex();
	virtual ~CIntelHex();

// helper
public:
	VOID UINT8ToHex(UINT8 val, INT8 *data);
	UINT8 HexToUINT8(INT8 *data);

	VOID UINT16ToHex(UINT16 val, INT8 *data);
	UINT16 HexToUINT16(INT8 *data);

	INT32 MakeCrc(INT8 *data);
	INT32 CheckCrc(INT8 *data);

public:
	UINT8 GetCount(INT8 *data);
	UINT16 GetAddress(INT8 *data);
	UINT8 GetType(INT8 *data);
	UINT8 GetData(INT8 *data, UINT8 *dest);
	UINT8 GetData8(INT8 *data, UINT8 num);
	UINT16 GetData16(INT8 *data, UINT8 num);
	virtual INT32 MakeTypedDataLine(UINT16 address, UINT8 type, UINT8 *buf, UINT8 length, INT8 *data);
	virtual INT32 MakeDataLine(UINT16 address, UINT8 *buf, UINT8 length, INT8 *data);
	virtual INT32 MakeEndLine(INT8 *data);
	INT32 CheckFile(INT8 *fileName);
};

#endif

